@@ -17,6 +17,8 @@
 
 #include "ep.h"
 
+
+
 /*
 /* Character Translation
 */
@@ -287,6 +289,270 @@ struct tCharTrans Char2Html [] =
         { '\255', "&yuml;"   },    /* 	Small y, diresis / umlaut  */
     } ; 
  
+
+struct tCharTrans Char2Url [] = 
+
+    {
+        { ' ' ,   "%00"         },    /* &#00;		Unused */ 
+        { ' ' ,   "%01"         },    /* &#01;		Unused */
+        { ' ' ,   "%02"         },    /* &#02;		Unused  */
+        { ' ' ,   "%03"         },    /* &#03;		Unused  */
+        { ' ' ,   "%04"         },    /* &#04;		Unused  */
+        { ' ' ,   "%05"         },    /* &#05;		Unused  */
+        { ' ' ,   "%06"         },    /* &#06;		Unused  */
+        { ' ' ,   "%07"         },    /* &#07;		Unused  */
+        { ' ' ,   "%08"         },    /* &#08;		Unused  */
+        { ' ' ,   "%09"         },    /* &#09;		Horizontal tab  */
+        { ' ' ,   "%0A"         },    /* &#10;		Line feed  */
+        { ' ' ,   "%0B"         },    /* &#11;		Unused  */
+        { ' ' ,   "%0C"         },    /* &#12;		Unused  */
+        { ' ' ,   "%0D"         },    /* &#13;		Carriage Return  */
+        { ' ' ,   "%0E"         },    /* &#14;		Unused  */
+        { ' ' ,   "%0F"         },    /* &#15;		Unused  */
+        { ' ' ,   "%10"         },    /* &#16;		Unused  */
+        { ' ' ,   "%11"         },    /* &#17;		Unused  */
+        { ' ' ,   "%12"         },    /* &#18;		Unused  */
+        { ' ' ,   "%13"         },    /* &#19;		Unused  */
+        { ' ' ,   "%14"         },    /* &#20;		Unused  */
+        { ' ' ,   "%15"         },    /* &#21;		Unused  */
+        { ' ' ,   "%16"         },    /* &#22;		Unused  */
+        { ' ' ,   "%17"         },    /* &#23;		Unused  */
+        { ' ' ,   "%18"         },    /* &#24;		Unused  */
+        { ' ' ,   "%19"         },    /* &#25;		Unused  */
+        { ' ' ,   "%1A"         },    /* &#26;		Unused  */
+        { ' ' ,   "%1B"         },    /* &#27;		Unused  */
+        { ' ' ,   "%1C"         },    /* &#28;		Unused  */
+        { ' ' ,   "%1D"         },    /* &#29;		Unused  */
+        { ' ' ,   "%1E"         },    /* &#30;		Unused  */
+        { ' ' ,   "%1F"         },    /* &#31;		Unused  */
+        { ' ' ,   "+"           },    /* 	&#32;		Space  */
+        { '!' ,   "%21"         },    /* 	&#33;		Exclamation mark  */
+        { '"' ,   "%22"   },    /* 	Quotation mark  */
+        { '#' ,   "%23"         },    /* 	&#35;		Number sign  */
+        { '$' ,   "%24"         },    /* 	&#36;		Dollar sign  */
+        { '%' ,   "%25"         },    /* 	&#37;		Percent sign  */
+        { '&' ,   "%26"    },    /* 	Ampersand  */
+        { '\'' ,  "%27"         },    /* 	&#39;		Apostrophe  */
+        { '(' ,   ""         },    /* 	&#40;		Left parenthesis  */
+        { ')' ,   ""         },    /* 	&#41;		Right parenthesis  */
+        { '*' ,   "%2A"         },    /* 	&#42;		Asterisk  */
+        { '+' ,   "%2B"         },    /* 	&#43;		Plus sign  */
+        { ',' ,   "%2C"         },    /* 	&#44;		Comma  */
+        { '-' ,   "%2D"         },    /* 	&#45;		Hyphen  */
+        { '.' ,   ""         },    /* 	&#46;		Period (fullstop)  */
+        { '/' ,   ""         },    /* 	&#47;		Solidus (slash)  */
+        { '0' ,   ""         },    /* 	&#48;		Digit 0  */
+        { '1' ,   ""         },    /* 	&#49;		Digit 1  */
+        { '2' ,   ""         },    /* 	&#50;		Digit 2  */
+        { '3' ,   ""         },    /* 	&#51;		Digit 3  */
+        { '4' ,   ""         },    /* 	&#52;		Digit 4  */
+        { '5' ,   ""         },    /* 	&#53;		Digit 5  */
+        { '6' ,   ""         },    /* 	&#54;		Digit 6  */
+        { '7' ,   ""         },    /* 	&#55;		Digit 7  */
+        { '8' ,   ""         },    /* 	&#56;		Digit 8  */
+        { '9' ,   ""         },    /* 	&#57;		Digit 9  */
+        { ':' ,   ""         },    /* 	&#58;		Colon  */
+        { ';' ,   "%3B"      },    /* 	&#59;		Semicolon  */
+        { '<' ,   "%3C"      },    /* 	Less than  */
+        { '=' ,   "%3D"      },    /* 	&#61;		Equals sign  */
+        { '>' ,   "%3E"      },    /* 	Greater than  */
+        { '?' ,   "%3F"      },    /* 	&#63;		Question mark  */
+        { '@' ,   "%40"      },    /* 	&#64;		Commercial at  */
+        { 'A' ,   ""         },    /* 	&#65;		Capital A  */
+        { 'B' ,   ""         },    /* 	&#66;		Capital B  */
+        { 'C' ,   ""         },    /* 	&#67;		Capital C  */
+        { 'D' ,   ""         },    /* 	&#68;		Capital D  */
+        { 'E' ,   ""         },    /* 	&#69;		Capital E  */
+        { 'F' ,   ""         },    /* 	&#70;		Capital F  */
+        { 'G' ,   ""         },    /* 	&#71;		Capital G  */
+        { 'H' ,   ""         },    /* 	&#72;		Capital H  */
+        { 'I' ,   ""         },    /* 	&#73;		Capital I  */
+        { 'J' ,   ""         },    /* 	&#74;		Capital J  */
+        { 'K' ,   ""         },    /* 	&#75;		Capital K  */
+        { 'L' ,   ""         },    /* 	&#76;		Capital L  */
+        { 'M' ,   ""         },    /* 	&#77;		Capital M  */
+        { 'N' ,   ""         },    /* 	&#78;		Capital N  */
+        { 'O' ,   ""         },    /* 	&#79;		Capital O  */
+        { 'P' ,   ""         },    /* 	&#80;		Capital P  */
+        { 'Q' ,   ""         },    /* 	&#81;		Capital Q  */
+        { 'R' ,   ""         },    /* 	&#82;		Capital R  */
+        { 'S' ,   ""         },    /* 	&#83;		Capital S  */
+        { 'T' ,   ""         },    /* 	&#84;		Capital T  */
+        { 'U' ,   ""         },    /* 	&#85;		Capital U  */
+        { 'V' ,   ""         },    /* 	&#86;		Capital V  */
+        { 'W' ,   ""         },    /* 	&#87;		Capital W  */
+        { 'X' ,   ""         },    /* 	&#88;		Capital X  */
+        { 'Y' ,   ""         },    /* 	&#89;		Capital Y  */
+        { 'Z' ,   ""         },    /* 	&#90;		Capital Z  */
+        { '[' ,   ""         },    /* 	&#91;		Left square bracket  */
+        { '\\' ,  "%7C"         },    /* 	&#92;		Reverse solidus (backslash)  */
+        { ']' ,   ""         },    /* 	&#93;		Right square bracket  */
+        { '^' ,   "%7E"         },    /* 	&#94;		Caret  */
+        { '_' ,   "%7F"         },    /* 	&#95;		Horizontal bar (underscore)  */
+        { '`' ,   ""         },    /* 	&#96;		Acute accent  */
+        { 'a' ,   ""         },    /* 	&#97;		Small a  */
+        { 'b' ,   ""         },    /* 	&#98;		Small b  */
+        { 'c' ,   ""         },    /* 	&#99;		Small c  */
+        { 'd' ,   ""         },    /* 	&#100;		Small d  */
+        { 'e' ,   ""         },    /* 	&#101;		Small e  */
+        { 'f' ,   ""         },    /* 	&#102;		Small f  */
+        { 'g' ,   ""         },    /* 	&#103;		Small g  */
+        { 'h' ,   ""         },    /* 	&#104;		Small h  */
+        { 'i' ,   ""         },    /* 	&#105;		Small i  */
+        { 'j' ,   ""         },    /* 	&#106;		Small j  */
+        { 'k' ,   ""         },    /* 	&#107;		Small k  */
+        { 'l' ,   ""         },    /* 	&#108;		Small l  */
+        { 'm' ,   ""         },    /* 	&#109;		Small m  */
+        { 'n' ,   ""         },    /* 	&#110;		Small n  */
+        { 'o' ,   ""         },    /* 	&#111;		Small o  */
+        { 'p' ,   ""         },    /* 	&#112;		Small p  */
+        { 'q' ,   ""         },    /* 	&#113;		Small q  */
+        { 'r' ,   ""         },    /* 	&#114;		Small r  */
+        { 's' ,   ""         },    /* 	&#115;		Small s  */
+        { 't' ,   ""         },    /* 	&#116;		Small t  */
+        { 'u' ,   ""         },    /* 	&#117;		Small u  */
+        { 'v' ,   ""         },    /* 	&#118;		Small v  */
+        { 'w' ,   ""         },    /* 	&#119;		Small w  */
+        { 'x' ,   ""         },    /* 	&#120;		Small x  */
+        { 'y' ,   ""         },    /* 	&#121;		Small y  */
+        { 'z' ,   ""         },    /* 	&#122;		Small z  */
+        { '{' ,   ""         },    /* 	&#123;		Left curly brace  */
+        { '|' ,   ""         },    /* 	&#124;		Vertical bar  */
+        { '}' ,   ""         },    /* 	&#125;		Right curly brace  */
+        { '~' ,   ""         },    /* 	&#126;		Tilde  */
+        { '' ,   ""         },    /* 	&#127;		Unused  */
+        { '' ,   ""         },    /* 	&#128;		Unused */
+         
+        { ' ' ,   ""         },    /* &#129;		Unused  */
+        { ' ' ,   ""         },    /* &#130;		Unused  */
+        { ' ' ,   ""         },    /* &#131;		Unused  */
+        { ' ' ,   ""         },    /* &#132;		Unused  */
+        { ' ' ,   ""         },    /* &#133;		Unused  */
+        { ' ' ,   ""         },    /* &#134;		Unused  */
+        { ' ' ,   ""         },    /* &#135;		Unused  */
+        { ' ' ,   ""         },    /* &#136;		Unused  */
+        { ' ' ,   ""         },    /* &#137;		Unused  */
+        { ' ' ,   ""         },    /* &#138;		Horizontal tab  */
+        { ' ' ,   ""         },    /* &#139;		Line feed  */
+        { ' ' ,   ""         },    /* &#140;		Unused  */
+        { ' ' ,   ""         },    /* &#141;		Unused  */
+        { ' ' ,   ""         },    /* &#142;		Carriage Return  */
+        { ' ' ,   ""         },    /* &#143;		Unused  */
+        { ' ' ,   ""         },    /* &#144;		Unused  */
+        { ' ' ,   ""         },    /* &#145;		Unused  */
+        { ' ' ,   ""         },    /* &#146;		Unused  */
+        { ' ' ,   ""         },    /* &#147;		Unused  */
+        { ' ' ,   ""         },    /* &#148;		Unused  */
+        { ' ' ,   ""         },    /* &#149;		Unused  */
+        { ' ' ,   ""         },    /* &#150;		Unused  */
+        { ' ' ,   ""         },    /* &#151;		Unused  */
+        { ' ' ,   ""         },    /* &#152;		Unused  */
+        { ' ' ,   ""         },    /* &#153;		Unused  */
+        { ' ' ,   ""         },    /* &#154;		Unused  */
+        { ' ' ,   ""         },    /* &#155;		Unused  */
+        { ' ' ,   ""         },    /* &#156;		Unused  */
+        { ' ' ,   ""         },    /* &#157;		Unused  */
+        { ' ' ,   ""         },    /* &#158;		Unused  */
+        { ' ' ,   ""         },    /* &#159;		Unused  */
+        { '' ,   ""   },    /* 	Non-breaking Space  */
+        { '' ,   ""  },    /* 	Inverted exclamation    */
+        { '' ,   ""   },    /* 	Cent sign               */
+        { '' ,   ""  },    /* 	Pound sterling  */
+        { '' ,   "" },    /* 	General currency sign  */
+        { '' ,   ""    },    /* 	Yen sign  */
+        { '' ,   "" },    /* 	Broken vertical bar  */
+        { '' ,   ""  },    /* 	Section sign  */
+        { '' ,   ""   },    /* 	Diresis / Umlaut  */
+        { '' ,   ""  },    /* 	Copyright               */
+        { '' ,   ""  },    /* 	Feminine ordinal  */
+        { '' ,   "" },    /* 	Left angle quote, guillemot left  */
+        { '' ,   ""   },    /*	Not sign  */
+        { '' ,   ""    },    /* 	Soft hyphen  */
+        { '' ,   ""    },    /* 	Registered trademark  */
+        { '' ,   ""  },    /* 	Macron accent  */
+        { '' ,   ""    },    /* 	Degree sign  */
+        { '' ,   "" },    /* 	Plus or minus  */
+        { '' ,   ""   },    /* 	Superscript two  */
+        { '' ,   ""   },    /* 	Superscript three  */
+        { '' ,   ""  },    /* 	Acute accent  */
+        { '' ,   ""  },    /* 	Micro sign  */
+        { '' ,   ""   },    /* 	Paragraph sign  */
+        { '' ,   "" },    /* 	Middle dot  */
+        { '' ,   ""  },    /* 	Cedilla  */
+        { '' ,   ""   },    /* 	Superscript one  */
+        { '' ,   ""   },    /* 	Masculine ordinal  */
+        { '' ,   ""  },    /* 	Right angle quote, guillemot right  */
+        { '' ,   "" },    /* 	Fraction one-fourth  */
+        { '' ,   "" },    /* 	Fraction one-half  */
+        { '' ,   "" },    /* 	Fraction three-fourths  */
+        { '' ,   "" },    /* 	Inverted question mark  */
+        { '' ,   "" },    /* 	Capital A, grave accent  */
+        { '' ,   "" },    /* 	Capital A, acute accent  */
+        { '' ,   ""  },    /* 	Capital A, circumflex  */
+        { '' ,   "" },    /* 	Capital A, tilde  */
+        { '' ,   ""   },    /* 	Capital A, diresis / umlaut  */
+        { '' ,   ""  },    /* 	Capital A, ring  */
+        { '' ,   ""  },    /* 	Capital AE ligature  */
+        { '' ,   "" },    /* 	Capital C, cedilla  */
+        { '' ,   "" },    /* 	Capital E, grave accent  */
+        { '' ,   "" },    /* 	Capital E, acute accent  */
+        { '' ,   ""  },    /* 	Capital E, circumflex  */
+        { '' ,   ""   },    /* 	Capital E, diresis / umlaut  */
+        { '' ,   "" },    /* 	Capital I, grave accent  */
+        { '' ,   "" },    /* 	Capital I, acute accent  */
+        { '' ,   ""  },    /* 	Capital I, circumflex  */
+        { '' ,   ""   },    /* 	Capital I, diresis / umlaut  */
+        { '' ,   ""    },    /* 	Capital Eth, Icelandic  */
+        { '' ,   "" },    /* 	Capital N, tilde  */
+        { '' ,   "" },    /* 	Capital O, grave accent  */
+        { '' ,   "" },    /* 	Capital O, acute accent  */
+        { '' ,   ""  },    /* 	Capital O, circumflex  */
+        { '' ,   "" },    /* 	Capital O, tilde  */
+        { '' ,   ""   },    /* 	Capital O, diresis / umlaut  */
+        { '' ,   ""  },    /* 	Multiply sign  */
+        { '' ,   "" },    /* 	Capital O, slash  */
+        { '' ,   "" },    /* 	Capital U, grave accent  */
+        { '' ,   "" },    /* 	Capital U, acute accent  */
+        { '' ,   ""  },    /* 	Capital U, circumflex  */
+        { '' ,   ""   },    /* 	Capital U, diresis / umlaut  */
+        { '' ,   "" },    /* 	Capital Y, acute accent  */
+        { '' ,   ""  },    /* 	Capital Thorn, Icelandic  */
+        { '' ,   ""  },    /* 	Small sharp s, German sz  */
+        { '' ,   "" },    /* 	Small a, grave accent  */
+        { '' ,   "" },    /* 	Small a, acute accent  */
+        { '' ,   ""  },    /* 	Small a, circumflex  */
+        { '' ,   "" },    /* 	Small a, tilde  */
+        { '' ,   ""   },    /* 	Small a, diresis / umlaut  */
+        { '' ,   ""  },    /* 	Small a, ring  */
+        { '' ,   ""  },    /* 	Small ae ligature  */
+        { '' ,   "" },    /* 	Small c, cedilla  */
+        { '' ,   "" },    /* 	Small e, grave accent  */
+        { '' ,   "" },    /* 	Small e, acute accent  */
+        { '' ,   ""  },    /* 	Small e, circumflex  */
+        { '' ,   ""   },    /* 	Small e, diresis / umlaut  */
+        { '' ,   "" },    /* 	Small i, grave accent  */
+        { '' ,   "" },    /* 	Small i, acute accent  */
+        { '' ,   ""  },    /* 	Small i, circumflex  */
+        { '' ,   ""   },    /* 	Small i, diresis / umlaut  */
+        { '' ,   ""    },    /* 	Small eth, Icelandic  */
+        { '' ,   "" },    /* 	Small n, tilde  */
+        { '' ,   "" },    /* 	Small o, grave accent  */
+        { '' ,   "" },    /* 	Small o, acute accent  */
+        { '' ,   ""  },    /* 	Small o, circumflex  */
+        { '' ,   "" },    /* 	Small o, tilde  */
+        { '' ,   ""   },    /* 	Small o, diresis / umlaut  */
+        { '' ,   "" },    /* 	Division sign  */
+        { '' ,   "" },    /* 	Small o, slash  */
+        { '' ,   "" },    /* 	Small u, grave accent  */
+        { '' ,   "" },    /* 	Small u, acute accent  */
+        { '' ,   ""  },    /* 	Small u, circumflex  */
+        { '' ,   ""   },    /* 	Small u, diresis / umlaut  */
+        { '' ,   "" },    /* 	Small y, acute accent  */
+        { '' ,   ""  },    /* 	Small thorn, Icelandic  */
+        { '\255', ""   },    /* 	Small y, diresis / umlaut  */
+    } ; 
+    
+    
 struct tCharTrans Html2Char [] = 
  
     {
@@ -398,3 +664,6 @@ struct tCharTrans Html2Char [] =
 
 int sizeHtml2Char = sizeof (Html2Char) / sizeof (struct tCharTrans) ;
 
+struct tCharTrans * pCurrEscape = Char2Html ;
+
+int bEscMode = escUrl | escHtml ;
